__version__='$Revision: 1.1 $'[11:-2]

import string

import ZODB
import AccessControl
import Acquisition
from DocumentTemplate.DT_Util import TemplateDict
from Globals import HTMLFile, HTML, MessageDialog
import OFS.SimpleItem
from Persistence import Persistent

from ZopeContext import ZopeContext


manage_addPresentationAgentForm = HTMLFile('agentAdd', globals())

def _get_class(classPath):
    i = string.rfind(classPath, '.')
    if i == -1:
        raise ValueError, "Class path must specify a module to import from", classPath
    module = classPath[0:i]
    className = classPath[i+1:]
    locals = {}
    exec "from %s import %s\n" % (module, className) in {}, locals
    return locals[className]


def manage_addPresentationAgent(self, id, title, presentationClassPath, urlArgumentNames, REQUEST=None):
    """Add a Presentation Agent"""

    agent = PresentationAgent(id, title, presentationClassPath, urlArgumentNames)
    self._setObject(id, agent)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

class PresentationAgent (OFS.SimpleItem.Item, Persistent, Acquisition.Implicit,
                         AccessControl.Role.RoleManager):

    meta_type = 'Presentation Agent'

    HelpSys = Acquisition.Acquired

    manage_options = (({'label': 'Properties', 'action': 'manage_main'},)
                      + OFS.SimpleItem.Item.manage_options
                      + AccessControl.Role.RoleManager.manage_options
                      )

    __ac_permissions__ = (
        ('View management screens', ('manage_main', 'manage_edit')),
        ('View', ('__getitem__', 'dispatch')),
        )

    def __init__(self, id, title, presentationClassPath, urlArgumentNames):
        self.id = id
        self.manage_edit(title, presentationClassPath, urlArgumentNames)

    def presentationClassPath(self):
        return self._presentationClassPath
    
    def urlArgumentNames(self):
        return self._urlArgumentNames

    def displayUrlArgumentNames(self):
        return string.join(self.urlArgumentNames(), '/')
    
    manage_main = HTMLFile('agentEdit', globals())

    def manage_edit(self, title, presentationClassPath, urlArgumentNames, REQUEST=None):
        "doc string"
        self.title = title
        self._presentationClassPath = presentationClassPath
        self._urlArgumentNames = string.split(urlArgumentNames, '/')
        if REQUEST:
            return MessageDialog(
                title = 'Changed %s' % self.id,
                message = '%s has been updated' % self.id,
                action = REQUEST['URL1'] + '/manage_main',
                target = 'manage_main')

    def __getitem__(self, item):
        """doc string"""

        print "PresentationAgent __getitem__", item
        
        if hasattr(self.REQUEST, '_PresentationAgent_urlIndex'):
            urlIndex = self.REQUEST._PresentationAgent_urlIndex
        else:
            urlIndex = 0

        if len(self._urlArgumentNames) > urlIndex:
            
            if hasattr(self.REQUEST, '_PresentationAgent_urlArguments'):
                urlArguments = self.REQUEST._PresentationAgent_urlArguments
            else:
                urlArguments = []
        
            urlArguments.append(item)
            urlIndex = urlIndex + 1

            self.REQUEST._PresentationAgent_urlIndex = urlIndex
            self.REQUEST._PresentationAgent_urlArguments = urlArguments
            
            return self
        
        self.REQUEST._PresentationAgent_methodName = item
        return self.dispatch
    

    _createNamespace = HTML('<dtml-return "_">')
    

    def dispatch(self):
        """doc string"""

        print "PresentationAgent dispatch"

        methodName = self.REQUEST._PresentationAgent_methodName
        delattr(self.REQUEST, '_PresentationAgent_methodName')
        
        print methodName
        
        if hasattr(self.REQUEST, '_PresentationAgent_urlArguments'):
            urlArguments = self.REQUEST._PresentationAgent_urlArguments
            delattr(self.REQUEST, '_PresentationAgent_urlArguments')
        else:
            urlArguments = []
            
        if hasattr(self.REQUEST, '_PresentationAgent_urlIndex'):
            delattr(self.REQUEST, '_PresentationAgent_urlIndex')
            
        print "my urlArguments are", urlArguments
        
        parentFolder = self.aq_parent
    
        namespace = self._createNamespace(None, self.REQUEST)
        namespace._push(parentFolder)

        context = ZopeContext(self.REQUEST, namespace, parentFolder)

        presentationClass = _get_class(self.presentationClassPath())
        presentation = apply(presentationClass, [context] + urlArguments)

        try:
            method = getattr(presentation, methodName)
        except AttributeError:
            raise AttributeError, ("Presentation class '" + str(presentationClass) +
                                   "' does not have a method named " + `methodName`)
        return method()
