__version__='$Revision: 1.1 $'[11:-2]

class ZopeContextMixin:
    
    def _initializeZopeContext(self, request, namespace, parentFolder):
        self._zopeContext_request = request
        self._zopeContext_namespace = namespace
        self._zopeContext_parentFolder = parentFolder

    def _copyZopeContext(self, zopeContext):
        self._initializeZopeContext(zopeContext.request(), zopeContext.namespace(),
                                    zopeContext.parentFolder())
        
    def request(self):
        "Returns the REQUEST object of information about the incoming web request."
        return self._zopeContext_request

    def response(self):
        """The RESPONSE object of how the web request is being answered."""
        return self.request().RESPONSE

    def namespace(self):
        "Returns the DTML namespace object, the one you access with the _ variable in DTML."

        return self._zopeContext_namespace

    def namespaceHasItem(self, name):
        """Queries if the DTML namespace has the named item, and returns true if so."""
        return self.namespace().has_key(name)
    
    def namespaceItem(self, name):
        """Retrieves the named item from the DTML namespace.  Even if
        callable, the object is not called automatically."""

        return self.namespace().getitem(name, 0)
    
    def namespaceCall(self, name):
        """Retrieves the named item from the DTML namespace.  If the
        object is callable, calls the object, and returns the result."""

        return self.namespace().getitem(name, 1)

    def namespaceWith(self, obj, function):

        """Similar to <dtml-with obj>, first pushes 'obj' onto the
        DTML namespace, and then calls the passed function.  The
        object is popped off the namespace, and the result of the
        function call is returned."""

        self.namespace()._push(obj)
        try:
            return function()
        finally:
            self.namespace()._pop(1)
            

    def parentFolder(self):
        """The folder containing the persistent object Zope traversed to
        following the URL.  This is useful to get the acquisition
        context of the web request."""

        return self._zopeContext_parentFolder
    
    def rootFolder(self):
        """Returns the root folder of the Zope application."""
        return self.request().PARENTS[-1]

    def call(self, dtml):
        """Calls the DTML object (usually a DTML Document or Method)
        with the DTML namespace, and returns the result."""

        return dtml(self, self.namespace())

    def form(self):
        """Returns the dictionary of form request variables."""
        return self.request().form

    def formItem(self, item):
        """Retrieves the named form request variable."""
        return self.form()[item]
    


class ZopeContext (ZopeContextMixin):
    def __init__(self, request, namespace, parentFolder):
        self._initializeZopeContext(request, namespace, parentFolder)
